using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using Orleans;
using Orleans.Configuration;
using Orleans.Hosting;
using OrleansDockerTest.GrainTypes;
using System.Threading.Tasks;
using IHostingEnvironment = Microsoft.AspNetCore.Hosting.IHostingEnvironment;

namespace OrleansDockerTest.Web
{
    public class Startup
    {
        public Startup(IConfiguration configuration)
        {
            Configuration = configuration;
        }

        public IConfiguration Configuration { get; }

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {
            services.AddMvc().SetCompatibilityVersion(CompatibilityVersion.Version_2_1);

            services.AddTransient(typeof(IClientBuilder), (p) =>
            {
                var connectionString = "Host=postgredb; Port=5432; Username=postgres; Password=1234qqqQ; Database=Apostil; Pooling=true;";
                var adoProvider = "Npgsql";

                IClientBuilder clientBuilder = new ClientBuilder()
                    .Configure<ClusterOptions>(options =>
                    {
                        options.ClusterId = "Cluster";
                        options.ServiceId = "OrleansDockerTest";
                    })
                    .ConfigureLogging(c =>
                    {
                        c.AddConsole();
                    })
                    .ConfigureApplicationParts(parts => parts.AddApplicationPart(typeof(ITestGrain).Assembly).WithReferences())
                    .UseAdoNetClustering(options =>
                    {
                        options.ConnectionString = connectionString;
                        options.Invariant = adoProvider;
                    });

                return clientBuilder;
            });

            services.AddSingleton(typeof(IClusterClient), (p) => { 
                IClientBuilder builder = p.GetRequiredService<IClientBuilder>();

                var orleansClient = builder.Build();

                int retryCount = 0;

                orleansClient.Connect(async e =>
                {
                    await Task.Delay(1000);
                    retryCount++;
                    return retryCount == 3;
                }).Wait();

                return orleansClient;
            });
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IHostingEnvironment env)
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }

            app.UseMvc();
        }
    }
}
